package org.openmodelica.modelicaml.modelica.importer.handlers;

import java.util.Iterator;
import java.util.List;

import javax.lang.model.type.PrimitiveType;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.emf.common.command.Command;
import org.eclipse.emf.common.command.CompoundCommand;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.transaction.RecordingCommand;
import org.eclipse.emf.transaction.TransactionalEditingDomain;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.papyrus.core.services.ServiceException;
import org.eclipse.papyrus.core.services.ServicesRegistry;
import org.eclipse.papyrus.core.utils.BusinessModelResolver;
import org.eclipse.papyrus.core.utils.ServiceUtils;
import org.eclipse.papyrus.core.utils.ServiceUtilsForActionHandlers;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;
import org.eclipse.uml2.uml.Class;
import org.eclipse.uml2.uml.Element;
import org.eclipse.uml2.uml.Enumeration;
import org.eclipse.uml2.uml.NamedElement;
import org.eclipse.uml2.uml.Stereotype;
import org.openmodelica.modelicaml.common.constants.Constants;

public class DisconnectFromCodeSyncHandler extends AbstractHandler {
	private EObject selectedElement = null;



	public Object execute(ExecutionEvent event) throws ExecutionException {
		
		String msg = "This helper will now unapply the <<ModelicaModelProxy>> stereotype from the selected element and all its nested elements. " +
				"This stereotype is used to identify elements that should be synchronized with the code in '"+Constants.folderName_code_sync+"' folder." +
				"\n\nYou can connect this sub-tree to code synchronization again at any time." +
				"\n\nDo you want to proceed?";
		
		boolean go = MessageDialog.openQuestion(new Shell(), "Disconnect From Code Synchronization Inoformation", msg);
		if (go) {
			selectedElement = (EObject) adaptSelectedElement(getCurrentSelections().get(0));
			
			if (selectedElement instanceof NamedElement && ((NamedElement)selectedElement).getAppliedStereotype(Constants.stereotypeQName_ModelicaModelProxy) != null) {
				unapplyProxyStereotype((NamedElement) selectedElement);
			}
			else {
				MessageDialog.openError(new Shell(), "Disconnect from Code Synchronization Error", "Not a valid selection.");
			}
		}
		return null;
	}

	private void unapplyProxyStereotype(NamedElement proxy){
		ServicesRegistry serviceRegistry;
		try {
			serviceRegistry = ServiceUtilsForActionHandlers.getInstance().getServiceRegistry();
			TransactionalEditingDomain  editingDomain = ServiceUtils.getInstance().getTransactionalEditingDomain(serviceRegistry);
			
			// execute 
			editingDomain.getCommandStack().execute(getCommand(editingDomain, proxy));

			
		} catch (ServiceException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	
	protected Command getCommand(TransactionalEditingDomain editingDomain, final NamedElement proxy) {
		CompoundCommand cc = new CompoundCommand("Unapplying the proxy stereotype");
		Command command = new RecordingCommand(editingDomain) {
			@Override
			protected void doExecute() {
				// unapply the stereotype for the selected element
				Stereotype sProxy = proxy.getAppliedStereotype(Constants.stereotypeQName_ModelicaModelProxy);
				if (sProxy != null) {
					proxy.unapplyStereotype(sProxy);
				}

				
				// get all contents, iterate and unapply the stereotype.
				Iterator<EObject> i = proxy.eAllContents();

				while (i.hasNext()) {
					EObject object = i.next() ;
					if (object instanceof Package || object instanceof Class || object instanceof Enumeration || object instanceof PrimitiveType) {
						Element nestedProxy = (Element)object;
						Stereotype s = nestedProxy.getAppliedStereotype(Constants.stereotypeQName_ModelicaModelProxy);

						if (s != null) {
							nestedProxy.unapplyStereotype(s);
						}
					}
				}
			}
		};
		cc.append(command);
		return (cc.unwrap());
	}
	
	
	
	public boolean isEnabled() {
		// TODO Auto-generated method stub
		return true;
	}

	
	/**
	 * Adapt selected element.
	 * 
	 * @param selection
	 *            the selection
	 * @return the e object
	 */
	protected EObject adaptSelectedElement( Object selection) {
		EObject eObject = null;
		if(selection != null) {
			if(selection instanceof IAdaptable) {
				selection = ((IAdaptable)selection).getAdapter(EObject.class);
			}
			Object businessObject = BusinessModelResolver.getInstance().getBusinessModel(selection);
			if(businessObject instanceof EObject) {
				eObject = (EObject)businessObject;
			}
		}
		return eObject;
	}
	
	private List<Object> getCurrentSelections() {
		ISelection selection = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getSelectionService().getSelection();
		if(selection instanceof IStructuredSelection) {
			IStructuredSelection structuredSelection = (IStructuredSelection)selection;
			return structuredSelection.toList();
		}
		return null;
	}
	

}
